/*
  Copyright (C) 2005-2013 Govert van Drimmelen

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.


  Govert van Drimmelen
  govert@icon.co.za
*/

#include "stdafx.h"
#include "ExcelDna.h"
#include "ExcelDnaLoader.h"

// Minimal parts of XLOPER types, 
// used only for xlAddInManagerInfo(12). Really.
struct XLOPER
{
	union
	{
		double num;					/* xltypeNum */
		void* str;					/* xltypeStr */
		WORD err;					/* xltypeErr */
	} val;
	WORD xltype; // Should be at offset 8 bytes
};

struct XLOPER12
{
	union
	{
		double num;					/* xltypeNum */
		void* str;					/* xltypeStr */
		int err;					/* xltypeErr */
		struct
		{
			double unused1;
			double unused2;
			double unused3;
		} unused;
	} val;
	DWORD xltype; // Should be at offset 24 bytes
};
#define xltypeNum  1;
#define xltypeStr  2;
#define xltypeErr  16;
#define xlerrValue 15

// The one and only ExportInfo
XlAddInExportInfo* pExportInfo = NULL;
// Flag to coordinate load/unload close and remove.
HMODULE lockModule;
bool locked = false;
bool removed = false;    // Used to check whether AutoRemove is called before AutoClose.
bool autoOpened = false; // Not set when loaded for COM server only. Used for re-open check.

// The actual thunk table 
extern "C" 
{
	PFN thunks[EXPORT_COUNT];
}

XlAddInExportInfo* CreateExportInfo()
{
	pExportInfo = new XlAddInExportInfo();
	pExportInfo->ExportInfoVersion = 7;
	pExportInfo->AppDomainId = -1;
	pExportInfo->pXlAutoOpen = NULL;
	pExportInfo->pXlAutoClose = NULL;
	pExportInfo->pXlAutoRemove = NULL;
	pExportInfo->pXlAutoFree = NULL;
	pExportInfo->pXlAutoFree12 = NULL;
	pExportInfo->pSetExcel12EntryPt = NULL;
	pExportInfo->pDllRegisterServer = NULL;
	pExportInfo->pDllUnregisterServer = NULL;
	pExportInfo->pDllGetClassObject = NULL;
	pExportInfo->pDllCanUnloadNow = NULL;
	pExportInfo->pSyncMacro = NULL;
	pExportInfo->pCalculationCanceled = NULL;
	pExportInfo->pCalculationEnded = NULL;
	pExportInfo->ThunkTableLength = EXPORT_COUNT;
	pExportInfo->ThunkTable = (PFN*)thunks;
	return pExportInfo;
}

// Safe to be called repeatedly, but not from multiple threads
short EnsureInitialized()
{
	short result = 0;
	if (pExportInfo != NULL)
	{
		result = 1;
	}
	else
	{
		XlAddInExportInfo* pExportInfoTemp = CreateExportInfo();
		result = XlLibraryInitialize(pExportInfoTemp);
		if (result)
		{
			pExportInfo	= pExportInfoTemp;
		}
	}
	return result;
}

// Called only when AutoClose is called after AutoRemove.
void Uninitialize()
{
	delete pExportInfo;
	pExportInfo = NULL;
	for (int i = 0; i < EXPORT_COUNT; i++)
	{
		thunks[i] = NULL;
	}
}

// Ensure that the library stays loaded.
// May be called many times, but should keep opened only until unlocked once.
void LockModule()
{
	if (!locked)
	{
		CPath xllPath(GetAddInFullPath());
		xllPath.StripPath();
		lockModule = LoadLibrary(xllPath);
		locked = true;
	}
}

// Allow the library to be unloaded.
void UnlockModule()
{
	if (locked)
	{
		FreeLibrary(lockModule);
		locked = false;
	}
}

// Standard DLL entry point.
BOOL __stdcall DllMain( HMODULE hModule,
						DWORD  ul_reason_for_call,
						LPVOID lpReserved
						)
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		LoaderInitialize(hModule);
		break;
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		LoaderUnload();
		break;
	}
	return TRUE;
}

extern "C"
{
	// Forward declares, since these are now called by AutoOpen.
	short __stdcall xlAutoClose();
	short __stdcall xlAutoRemove();

	// Excel Add-In standard exports
	short __stdcall xlAutoOpen()
	{
		short result = 0;

		// If we are loaded as an add-in already, then ensure re-load = AddInRemove + AutoClose + AutoOpen,
		// which mains a clean AppDomain for each load.
		if (autoOpened)
		{
			xlAutoRemove();
			xlAutoClose();
		}

		if (EnsureInitialized() && 
			pExportInfo->pXlAutoOpen != NULL)
		{
			result = pExportInfo->pXlAutoOpen();
			LockModule();
			// Set the 'removed' flag to false, which prevents AutoClose from actually unloading (or calling through to the add-in),
			// unless AutoRemove is called first (from the add-in manager, a host or the re-open sequence above).
			removed = false;
			// Keep track that we are loaded as an add-in, not just a COM or RTD server.
			// This allows us to re-open in a clean AppDomain, yet load COM server first then add-in without damage.
			autoOpened = true;
		}
		return result;
	}

	short __stdcall xlAutoClose()
	{
		short result = 0;
		if (EnsureInitialized() && 
			pExportInfo->pXlAutoClose != NULL)
		{
			result = pExportInfo->pXlAutoClose();
			if (removed)
			{
				// TODO: Consider how and when to unload
				//       Unloading the AppDomain could be a bit too dramatic if we are serving as a COM Server or RTD Server directly.
				// DOCUMENT: What the current implementation is.
				// No more managed functions should be called.
				Uninitialize();

				// Complete the clean-up by unloading AppDomain
				XlLibraryUnload();
				// ... recording that we are no longer open as an add-in.
				autoOpened = false;
				// ...and allowing the .xll itself to be unloaded
				UnlockModule();
			}
		}
		return result;
	}
	
	// Since v0.29 loading is much more expensive, so I want to reduce the number of times we load.
	// We've never used or exposed xlAutoAdd to Excel-DNA addins, so no harm in disabling for now.
	// To add back, also uncomment in the ExcelDna.def file.
	//short __stdcall xlAutoAdd()
	//{
	//	short result = 0;
	//	if (EnsureInitialized() && 
	//		pExportInfo->pXlAutoAdd != NULL)
	//	{
	//		result = pExportInfo->pXlAutoAdd();
	//	}
	//	return result;
	//}

	short __stdcall xlAutoRemove()
	{
		short result = 0;
		if (EnsureInitialized() && 
			pExportInfo->pXlAutoRemove != NULL)
		{
			result = pExportInfo->pXlAutoRemove();
			// Set the 'removed' flag which will allow the AutoClose to actually unload (and call through to the add-in).
			removed = true;
		}
		return result;
	}

	void __stdcall xlAutoFree(void* pXloper)
	{
		if (pExportInfo != NULL && pExportInfo->pXlAutoFree != NULL)
		{
			pExportInfo->pXlAutoFree(pXloper);
		}
	}

	void __stdcall xlAutoFree12(void* pXloper12)
	{
		if (pExportInfo != NULL && pExportInfo->pXlAutoFree12 != NULL)
		{
			pExportInfo->pXlAutoFree12(pXloper12);
		}
	}

	XLOPER* __stdcall xlAddInManagerInfo(XLOPER* pXloper)
	{
		static XLOPER result;
		static char name[256];

		// Return error by default
		result.xltype = xltypeErr;
		result.val.err = xlerrValue;

		if (pXloper->xltype == 1 && pXloper->val.num == 1.0)
		{
			CString addInNameW;
			HRESULT hr = GetAddInName(addInNameW);
			if (!FAILED(hr))
			{
				CStringA addInName(addInNameW);
				byte length = (byte)min(addInName.GetLength(), 255);
				name[0] = (char)length;
				const char* pAddInName = addInName;
				char* pName = (char*)name + 1;
				CStringA::CopyChars(pName, 255, pAddInName, length);

				result.xltype = xltypeStr;
				result.val.str = name;
			}
		}

		return &result;
	}

	XLOPER12* __stdcall xlAddInManagerInfo12(XLOPER12* pXloper)
	{
		static XLOPER12 result;
		static wchar_t name[256];

		// Return error by default
		result.xltype = xltypeErr;
		result.val.err = xlerrValue;

		if (pXloper->xltype == 1 && pXloper->val.num == 1.0)
		{
			CString addInName;
			HRESULT hr = GetAddInName(addInName);
			if (!FAILED(hr))
			{
				// We could probably use CString as is (maybe with truncation)!?
				int length = (int)min(addInName.GetLength(), 255);
				name[0] = (wchar_t)length;
				const wchar_t* pAddInName = addInName;
				wchar_t* pName = (wchar_t*)name + 1;
				CString::CopyChars(pName, 255, pAddInName, length);
				result.xltype = xltypeStr;
				result.val.str = name;
			}
		}

		return &result;
	}

	// Support for Excel 2010 SDK - used when loading under HPC XLL Host
	void __stdcall SetExcel12EntryPt(void* pexcel12New)
	{
		if (EnsureInitialized() && 
			pExportInfo->pSetExcel12EntryPt != NULL)
		{
			pExportInfo->pSetExcel12EntryPt(pexcel12New);
		}
	}

	// We are also a COM Server, to support the =RTD(...) worksheet function and VBA ComServer integration.
	HRESULT __stdcall DllRegisterServer()
	{
		HRESULT result = E_UNEXPECTED;
		if (EnsureInitialized() && 
			pExportInfo->pDllRegisterServer != NULL)
		{
			result = pExportInfo->pDllRegisterServer();
		}
		return result;
	}

	HRESULT __stdcall DllUnregisterServer()
	{
		HRESULT result = E_UNEXPECTED;
		if (EnsureInitialized() && 
			pExportInfo->pDllUnregisterServer != NULL)
		{
			result = pExportInfo->pDllUnregisterServer();
		}
		return result;
	}
	
	HRESULT __stdcall DllGetClassObject(REFCLSID clsid, REFIID iid, void** ppv)
	{
		HRESULT result = E_UNEXPECTED;
		GUID cls = clsid;
		GUID i = iid;
		if (EnsureInitialized() && 
			pExportInfo->pDllGetClassObject != NULL)
		{

			result = pExportInfo->pDllGetClassObject(cls, i, ppv);
		}
		return result;
	}

	HRESULT __stdcall DllCanUnloadNow()
	{
		HRESULT result = S_OK;
		if (EnsureInitialized() && 
			pExportInfo->pDllCanUnloadNow != NULL)
		{
			result = pExportInfo->pDllCanUnloadNow();
		}
		return result;
	}

	void __stdcall SyncMacro(double param)
	{
		if (EnsureInitialized() && 
			pExportInfo->pSyncMacro != NULL)
		{
			pExportInfo->pSyncMacro(param);
		}
	}

	void __stdcall CalculationCanceled()
	{
		if (EnsureInitialized() && 
			pExportInfo->pCalculationCanceled != NULL)
		{
			pExportInfo->pCalculationCanceled();
		}
	}

	void __stdcall CalculationEnded()
	{
		if (EnsureInitialized() && 
			pExportInfo->pCalculationEnded != NULL)
		{
			pExportInfo->pCalculationEnded();
		}
	}

}


#ifndef _M_X64
// The dll export implementation that jmps to thunk in the thunktable
// For x64 this is implemented in JmpExports64.asm

// Use extern so that functions are not decorated when exported.
// naked ensures no prologue or epilogue generated by the compiler 
// - jump directly to unmanaged thunk at offset i
#define expf(i) extern "C" __declspec(dllexport,naked) void f##i(void){	__asm jmp thunks + i * 4 /* sizeof(PFN) (only used on 32-bit) */ }

// Declare the functions -- NOTE: list here must go from 0 to EXPORT_COUNT-1
expf(0)
expf(1)
expf(2)
expf(3)
expf(4)
expf(5)
expf(6)
expf(7)
expf(8)
expf(9)
expf(10)
expf(11)
expf(12)
expf(13)
expf(14)
expf(15)
expf(16)
expf(17)
expf(18)
expf(19)
expf(20)
expf(21)
expf(22)
expf(23)
expf(24)
expf(25)
expf(26)
expf(27)
expf(28)
expf(29)
expf(30)
expf(31)
expf(32)
expf(33)
expf(34)
expf(35)
expf(36)
expf(37)
expf(38)
expf(39)
expf(40)
expf(41)
expf(42)
expf(43)
expf(44)
expf(45)
expf(46)
expf(47)
expf(48)
expf(49)
expf(50)
expf(51)
expf(52)
expf(53)
expf(54)
expf(55)
expf(56)
expf(57)
expf(58)
expf(59)
expf(60)
expf(61)
expf(62)
expf(63)
expf(64)
expf(65)
expf(66)
expf(67)
expf(68)
expf(69)
expf(70)
expf(71)
expf(72)
expf(73)
expf(74)
expf(75)
expf(76)
expf(77)
expf(78)
expf(79)
expf(80)
expf(81)
expf(82)
expf(83)
expf(84)
expf(85)
expf(86)
expf(87)
expf(88)
expf(89)
expf(90)
expf(91)
expf(92)
expf(93)
expf(94)
expf(95)
expf(96)
expf(97)
expf(98)
expf(99)
expf(100)
expf(101)
expf(102)
expf(103)
expf(104)
expf(105)
expf(106)
expf(107)
expf(108)
expf(109)
expf(110)
expf(111)
expf(112)
expf(113)
expf(114)
expf(115)
expf(116)
expf(117)
expf(118)
expf(119)
expf(120)
expf(121)
expf(122)
expf(123)
expf(124)
expf(125)
expf(126)
expf(127)
expf(128)
expf(129)
expf(130)
expf(131)
expf(132)
expf(133)
expf(134)
expf(135)
expf(136)
expf(137)
expf(138)
expf(139)
expf(140)
expf(141)
expf(142)
expf(143)
expf(144)
expf(145)
expf(146)
expf(147)
expf(148)
expf(149)
expf(150)
expf(151)
expf(152)
expf(153)
expf(154)
expf(155)
expf(156)
expf(157)
expf(158)
expf(159)
expf(160)
expf(161)
expf(162)
expf(163)
expf(164)
expf(165)
expf(166)
expf(167)
expf(168)
expf(169)
expf(170)
expf(171)
expf(172)
expf(173)
expf(174)
expf(175)
expf(176)
expf(177)
expf(178)
expf(179)
expf(180)
expf(181)
expf(182)
expf(183)
expf(184)
expf(185)
expf(186)
expf(187)
expf(188)
expf(189)
expf(190)
expf(191)
expf(192)
expf(193)
expf(194)
expf(195)
expf(196)
expf(197)
expf(198)
expf(199)
expf(200)
expf(201)
expf(202)
expf(203)
expf(204)
expf(205)
expf(206)
expf(207)
expf(208)
expf(209)
expf(210)
expf(211)
expf(212)
expf(213)
expf(214)
expf(215)
expf(216)
expf(217)
expf(218)
expf(219)
expf(220)
expf(221)
expf(222)
expf(223)
expf(224)
expf(225)
expf(226)
expf(227)
expf(228)
expf(229)
expf(230)
expf(231)
expf(232)
expf(233)
expf(234)
expf(235)
expf(236)
expf(237)
expf(238)
expf(239)
expf(240)
expf(241)
expf(242)
expf(243)
expf(244)
expf(245)
expf(246)
expf(247)
expf(248)
expf(249)
expf(250)
expf(251)
expf(252)
expf(253)
expf(254)
expf(255)
expf(256)
expf(257)
expf(258)
expf(259)
expf(260)
expf(261)
expf(262)
expf(263)
expf(264)
expf(265)
expf(266)
expf(267)
expf(268)
expf(269)
expf(270)
expf(271)
expf(272)
expf(273)
expf(274)
expf(275)
expf(276)
expf(277)
expf(278)
expf(279)
expf(280)
expf(281)
expf(282)
expf(283)
expf(284)
expf(285)
expf(286)
expf(287)
expf(288)
expf(289)
expf(290)
expf(291)
expf(292)
expf(293)
expf(294)
expf(295)
expf(296)
expf(297)
expf(298)
expf(299)
expf(300)
expf(301)
expf(302)
expf(303)
expf(304)
expf(305)
expf(306)
expf(307)
expf(308)
expf(309)
expf(310)
expf(311)
expf(312)
expf(313)
expf(314)
expf(315)
expf(316)
expf(317)
expf(318)
expf(319)
expf(320)
expf(321)
expf(322)
expf(323)
expf(324)
expf(325)
expf(326)
expf(327)
expf(328)
expf(329)
expf(330)
expf(331)
expf(332)
expf(333)
expf(334)
expf(335)
expf(336)
expf(337)
expf(338)
expf(339)
expf(340)
expf(341)
expf(342)
expf(343)
expf(344)
expf(345)
expf(346)
expf(347)
expf(348)
expf(349)
expf(350)
expf(351)
expf(352)
expf(353)
expf(354)
expf(355)
expf(356)
expf(357)
expf(358)
expf(359)
expf(360)
expf(361)
expf(362)
expf(363)
expf(364)
expf(365)
expf(366)
expf(367)
expf(368)
expf(369)
expf(370)
expf(371)
expf(372)
expf(373)
expf(374)
expf(375)
expf(376)
expf(377)
expf(378)
expf(379)
expf(380)
expf(381)
expf(382)
expf(383)
expf(384)
expf(385)
expf(386)
expf(387)
expf(388)
expf(389)
expf(390)
expf(391)
expf(392)
expf(393)
expf(394)
expf(395)
expf(396)
expf(397)
expf(398)
expf(399)
expf(400)
expf(401)
expf(402)
expf(403)
expf(404)
expf(405)
expf(406)
expf(407)
expf(408)
expf(409)
expf(410)
expf(411)
expf(412)
expf(413)
expf(414)
expf(415)
expf(416)
expf(417)
expf(418)
expf(419)
expf(420)
expf(421)
expf(422)
expf(423)
expf(424)
expf(425)
expf(426)
expf(427)
expf(428)
expf(429)
expf(430)
expf(431)
expf(432)
expf(433)
expf(434)
expf(435)
expf(436)
expf(437)
expf(438)
expf(439)
expf(440)
expf(441)
expf(442)
expf(443)
expf(444)
expf(445)
expf(446)
expf(447)
expf(448)
expf(449)
expf(450)
expf(451)
expf(452)
expf(453)
expf(454)
expf(455)
expf(456)
expf(457)
expf(458)
expf(459)
expf(460)
expf(461)
expf(462)
expf(463)
expf(464)
expf(465)
expf(466)
expf(467)
expf(468)
expf(469)
expf(470)
expf(471)
expf(472)
expf(473)
expf(474)
expf(475)
expf(476)
expf(477)
expf(478)
expf(479)
expf(480)
expf(481)
expf(482)
expf(483)
expf(484)
expf(485)
expf(486)
expf(487)
expf(488)
expf(489)
expf(490)
expf(491)
expf(492)
expf(493)
expf(494)
expf(495)
expf(496)
expf(497)
expf(498)
expf(499)
expf(500)
expf(501)
expf(502)
expf(503)
expf(504)
expf(505)
expf(506)
expf(507)
expf(508)
expf(509)
expf(510)
expf(511)
expf(512)
expf(513)
expf(514)
expf(515)
expf(516)
expf(517)
expf(518)
expf(519)
expf(520)
expf(521)
expf(522)
expf(523)
expf(524)
expf(525)
expf(526)
expf(527)
expf(528)
expf(529)
expf(530)
expf(531)
expf(532)
expf(533)
expf(534)
expf(535)
expf(536)
expf(537)
expf(538)
expf(539)
expf(540)
expf(541)
expf(542)
expf(543)
expf(544)
expf(545)
expf(546)
expf(547)
expf(548)
expf(549)
expf(550)
expf(551)
expf(552)
expf(553)
expf(554)
expf(555)
expf(556)
expf(557)
expf(558)
expf(559)
expf(560)
expf(561)
expf(562)
expf(563)
expf(564)
expf(565)
expf(566)
expf(567)
expf(568)
expf(569)
expf(570)
expf(571)
expf(572)
expf(573)
expf(574)
expf(575)
expf(576)
expf(577)
expf(578)
expf(579)
expf(580)
expf(581)
expf(582)
expf(583)
expf(584)
expf(585)
expf(586)
expf(587)
expf(588)
expf(589)
expf(590)
expf(591)
expf(592)
expf(593)
expf(594)
expf(595)
expf(596)
expf(597)
expf(598)
expf(599)
expf(600)
expf(601)
expf(602)
expf(603)
expf(604)
expf(605)
expf(606)
expf(607)
expf(608)
expf(609)
expf(610)
expf(611)
expf(612)
expf(613)
expf(614)
expf(615)
expf(616)
expf(617)
expf(618)
expf(619)
expf(620)
expf(621)
expf(622)
expf(623)
expf(624)
expf(625)
expf(626)
expf(627)
expf(628)
expf(629)
expf(630)
expf(631)
expf(632)
expf(633)
expf(634)
expf(635)
expf(636)
expf(637)
expf(638)
expf(639)
expf(640)
expf(641)
expf(642)
expf(643)
expf(644)
expf(645)
expf(646)
expf(647)
expf(648)
expf(649)
expf(650)
expf(651)
expf(652)
expf(653)
expf(654)
expf(655)
expf(656)
expf(657)
expf(658)
expf(659)
expf(660)
expf(661)
expf(662)
expf(663)
expf(664)
expf(665)
expf(666)
expf(667)
expf(668)
expf(669)
expf(670)
expf(671)
expf(672)
expf(673)
expf(674)
expf(675)
expf(676)
expf(677)
expf(678)
expf(679)
expf(680)
expf(681)
expf(682)
expf(683)
expf(684)
expf(685)
expf(686)
expf(687)
expf(688)
expf(689)
expf(690)
expf(691)
expf(692)
expf(693)
expf(694)
expf(695)
expf(696)
expf(697)
expf(698)
expf(699)
expf(700)
expf(701)
expf(702)
expf(703)
expf(704)
expf(705)
expf(706)
expf(707)
expf(708)
expf(709)
expf(710)
expf(711)
expf(712)
expf(713)
expf(714)
expf(715)
expf(716)
expf(717)
expf(718)
expf(719)
expf(720)
expf(721)
expf(722)
expf(723)
expf(724)
expf(725)
expf(726)
expf(727)
expf(728)
expf(729)
expf(730)
expf(731)
expf(732)
expf(733)
expf(734)
expf(735)
expf(736)
expf(737)
expf(738)
expf(739)
expf(740)
expf(741)
expf(742)
expf(743)
expf(744)
expf(745)
expf(746)
expf(747)
expf(748)
expf(749)
expf(750)
expf(751)
expf(752)
expf(753)
expf(754)
expf(755)
expf(756)
expf(757)
expf(758)
expf(759)
expf(760)
expf(761)
expf(762)
expf(763)
expf(764)
expf(765)
expf(766)
expf(767)
expf(768)
expf(769)
expf(770)
expf(771)
expf(772)
expf(773)
expf(774)
expf(775)
expf(776)
expf(777)
expf(778)
expf(779)
expf(780)
expf(781)
expf(782)
expf(783)
expf(784)
expf(785)
expf(786)
expf(787)
expf(788)
expf(789)
expf(790)
expf(791)
expf(792)
expf(793)
expf(794)
expf(795)
expf(796)
expf(797)
expf(798)
expf(799)
expf(800)
expf(801)
expf(802)
expf(803)
expf(804)
expf(805)
expf(806)
expf(807)
expf(808)
expf(809)
expf(810)
expf(811)
expf(812)
expf(813)
expf(814)
expf(815)
expf(816)
expf(817)
expf(818)
expf(819)
expf(820)
expf(821)
expf(822)
expf(823)
expf(824)
expf(825)
expf(826)
expf(827)
expf(828)
expf(829)
expf(830)
expf(831)
expf(832)
expf(833)
expf(834)
expf(835)
expf(836)
expf(837)
expf(838)
expf(839)
expf(840)
expf(841)
expf(842)
expf(843)
expf(844)
expf(845)
expf(846)
expf(847)
expf(848)
expf(849)
expf(850)
expf(851)
expf(852)
expf(853)
expf(854)
expf(855)
expf(856)
expf(857)
expf(858)
expf(859)
expf(860)
expf(861)
expf(862)
expf(863)
expf(864)
expf(865)
expf(866)
expf(867)
expf(868)
expf(869)
expf(870)
expf(871)
expf(872)
expf(873)
expf(874)
expf(875)
expf(876)
expf(877)
expf(878)
expf(879)
expf(880)
expf(881)
expf(882)
expf(883)
expf(884)
expf(885)
expf(886)
expf(887)
expf(888)
expf(889)
expf(890)
expf(891)
expf(892)
expf(893)
expf(894)
expf(895)
expf(896)
expf(897)
expf(898)
expf(899)
expf(900)
expf(901)
expf(902)
expf(903)
expf(904)
expf(905)
expf(906)
expf(907)
expf(908)
expf(909)
expf(910)
expf(911)
expf(912)
expf(913)
expf(914)
expf(915)
expf(916)
expf(917)
expf(918)
expf(919)
expf(920)
expf(921)
expf(922)
expf(923)
expf(924)
expf(925)
expf(926)
expf(927)
expf(928)
expf(929)
expf(930)
expf(931)
expf(932)
expf(933)
expf(934)
expf(935)
expf(936)
expf(937)
expf(938)
expf(939)
expf(940)
expf(941)
expf(942)
expf(943)
expf(944)
expf(945)
expf(946)
expf(947)
expf(948)
expf(949)
expf(950)
expf(951)
expf(952)
expf(953)
expf(954)
expf(955)
expf(956)
expf(957)
expf(958)
expf(959)
expf(960)
expf(961)
expf(962)
expf(963)
expf(964)
expf(965)
expf(966)
expf(967)
expf(968)
expf(969)
expf(970)
expf(971)
expf(972)
expf(973)
expf(974)
expf(975)
expf(976)
expf(977)
expf(978)
expf(979)
expf(980)
expf(981)
expf(982)
expf(983)
expf(984)
expf(985)
expf(986)
expf(987)
expf(988)
expf(989)
expf(990)
expf(991)
expf(992)
expf(993)
expf(994)
expf(995)
expf(996)
expf(997)
expf(998)
expf(999)
expf(1000)
expf(1001)
expf(1002)
expf(1003)
expf(1004)
expf(1005)
expf(1006)
expf(1007)
expf(1008)
expf(1009)
expf(1010)
expf(1011)
expf(1012)
expf(1013)
expf(1014)
expf(1015)
expf(1016)
expf(1017)
expf(1018)
expf(1019)
expf(1020)
expf(1021)
expf(1022)
expf(1023)
expf(1024)
expf(1025)
expf(1026)
expf(1027)
expf(1028)
expf(1029)
expf(1030)
expf(1031)
expf(1032)
expf(1033)
expf(1034)
expf(1035)
expf(1036)
expf(1037)
expf(1038)
expf(1039)
expf(1040)
expf(1041)
expf(1042)
expf(1043)
expf(1044)
expf(1045)
expf(1046)
expf(1047)
expf(1048)
expf(1049)
expf(1050)
expf(1051)
expf(1052)
expf(1053)
expf(1054)
expf(1055)
expf(1056)
expf(1057)
expf(1058)
expf(1059)
expf(1060)
expf(1061)
expf(1062)
expf(1063)
expf(1064)
expf(1065)
expf(1066)
expf(1067)
expf(1068)
expf(1069)
expf(1070)
expf(1071)
expf(1072)
expf(1073)
expf(1074)
expf(1075)
expf(1076)
expf(1077)
expf(1078)
expf(1079)
expf(1080)
expf(1081)
expf(1082)
expf(1083)
expf(1084)
expf(1085)
expf(1086)
expf(1087)
expf(1088)
expf(1089)
expf(1090)
expf(1091)
expf(1092)
expf(1093)
expf(1094)
expf(1095)
expf(1096)
expf(1097)
expf(1098)
expf(1099)
expf(1100)
expf(1101)
expf(1102)
expf(1103)
expf(1104)
expf(1105)
expf(1106)
expf(1107)
expf(1108)
expf(1109)
expf(1110)
expf(1111)
expf(1112)
expf(1113)
expf(1114)
expf(1115)
expf(1116)
expf(1117)
expf(1118)
expf(1119)
expf(1120)
expf(1121)
expf(1122)
expf(1123)
expf(1124)
expf(1125)
expf(1126)
expf(1127)
expf(1128)
expf(1129)
expf(1130)
expf(1131)
expf(1132)
expf(1133)
expf(1134)
expf(1135)
expf(1136)
expf(1137)
expf(1138)
expf(1139)
expf(1140)
expf(1141)
expf(1142)
expf(1143)
expf(1144)
expf(1145)
expf(1146)
expf(1147)
expf(1148)
expf(1149)
expf(1150)
expf(1151)
expf(1152)
expf(1153)
expf(1154)
expf(1155)
expf(1156)
expf(1157)
expf(1158)
expf(1159)
expf(1160)
expf(1161)
expf(1162)
expf(1163)
expf(1164)
expf(1165)
expf(1166)
expf(1167)
expf(1168)
expf(1169)
expf(1170)
expf(1171)
expf(1172)
expf(1173)
expf(1174)
expf(1175)
expf(1176)
expf(1177)
expf(1178)
expf(1179)
expf(1180)
expf(1181)
expf(1182)
expf(1183)
expf(1184)
expf(1185)
expf(1186)
expf(1187)
expf(1188)
expf(1189)
expf(1190)
expf(1191)
expf(1192)
expf(1193)
expf(1194)
expf(1195)
expf(1196)
expf(1197)
expf(1198)
expf(1199)
expf(1200)
expf(1201)
expf(1202)
expf(1203)
expf(1204)
expf(1205)
expf(1206)
expf(1207)
expf(1208)
expf(1209)
expf(1210)
expf(1211)
expf(1212)
expf(1213)
expf(1214)
expf(1215)
expf(1216)
expf(1217)
expf(1218)
expf(1219)
expf(1220)
expf(1221)
expf(1222)
expf(1223)
expf(1224)
expf(1225)
expf(1226)
expf(1227)
expf(1228)
expf(1229)
expf(1230)
expf(1231)
expf(1232)
expf(1233)
expf(1234)
expf(1235)
expf(1236)
expf(1237)
expf(1238)
expf(1239)
expf(1240)
expf(1241)
expf(1242)
expf(1243)
expf(1244)
expf(1245)
expf(1246)
expf(1247)
expf(1248)
expf(1249)
expf(1250)
expf(1251)
expf(1252)
expf(1253)
expf(1254)
expf(1255)
expf(1256)
expf(1257)
expf(1258)
expf(1259)
expf(1260)
expf(1261)
expf(1262)
expf(1263)
expf(1264)
expf(1265)
expf(1266)
expf(1267)
expf(1268)
expf(1269)
expf(1270)
expf(1271)
expf(1272)
expf(1273)
expf(1274)
expf(1275)
expf(1276)
expf(1277)
expf(1278)
expf(1279)
expf(1280)
expf(1281)
expf(1282)
expf(1283)
expf(1284)
expf(1285)
expf(1286)
expf(1287)
expf(1288)
expf(1289)
expf(1290)
expf(1291)
expf(1292)
expf(1293)
expf(1294)
expf(1295)
expf(1296)
expf(1297)
expf(1298)
expf(1299)
expf(1300)
expf(1301)
expf(1302)
expf(1303)
expf(1304)
expf(1305)
expf(1306)
expf(1307)
expf(1308)
expf(1309)
expf(1310)
expf(1311)
expf(1312)
expf(1313)
expf(1314)
expf(1315)
expf(1316)
expf(1317)
expf(1318)
expf(1319)
expf(1320)
expf(1321)
expf(1322)
expf(1323)
expf(1324)
expf(1325)
expf(1326)
expf(1327)
expf(1328)
expf(1329)
expf(1330)
expf(1331)
expf(1332)
expf(1333)
expf(1334)
expf(1335)
expf(1336)
expf(1337)
expf(1338)
expf(1339)
expf(1340)
expf(1341)
expf(1342)
expf(1343)
expf(1344)
expf(1345)
expf(1346)
expf(1347)
expf(1348)
expf(1349)
expf(1350)
expf(1351)
expf(1352)
expf(1353)
expf(1354)
expf(1355)
expf(1356)
expf(1357)
expf(1358)
expf(1359)
expf(1360)
expf(1361)
expf(1362)
expf(1363)
expf(1364)
expf(1365)
expf(1366)
expf(1367)
expf(1368)
expf(1369)
expf(1370)
expf(1371)
expf(1372)
expf(1373)
expf(1374)
expf(1375)
expf(1376)
expf(1377)
expf(1378)
expf(1379)
expf(1380)
expf(1381)
expf(1382)
expf(1383)
expf(1384)
expf(1385)
expf(1386)
expf(1387)
expf(1388)
expf(1389)
expf(1390)
expf(1391)
expf(1392)
expf(1393)
expf(1394)
expf(1395)
expf(1396)
expf(1397)
expf(1398)
expf(1399)
expf(1400)
expf(1401)
expf(1402)
expf(1403)
expf(1404)
expf(1405)
expf(1406)
expf(1407)
expf(1408)
expf(1409)
expf(1410)
expf(1411)
expf(1412)
expf(1413)
expf(1414)
expf(1415)
expf(1416)
expf(1417)
expf(1418)
expf(1419)
expf(1420)
expf(1421)
expf(1422)
expf(1423)
expf(1424)
expf(1425)
expf(1426)
expf(1427)
expf(1428)
expf(1429)
expf(1430)
expf(1431)
expf(1432)
expf(1433)
expf(1434)
expf(1435)
expf(1436)
expf(1437)
expf(1438)
expf(1439)
expf(1440)
expf(1441)
expf(1442)
expf(1443)
expf(1444)
expf(1445)
expf(1446)
expf(1447)
expf(1448)
expf(1449)
expf(1450)
expf(1451)
expf(1452)
expf(1453)
expf(1454)
expf(1455)
expf(1456)
expf(1457)
expf(1458)
expf(1459)
expf(1460)
expf(1461)
expf(1462)
expf(1463)
expf(1464)
expf(1465)
expf(1466)
expf(1467)
expf(1468)
expf(1469)
expf(1470)
expf(1471)
expf(1472)
expf(1473)
expf(1474)
expf(1475)
expf(1476)
expf(1477)
expf(1478)
expf(1479)
expf(1480)
expf(1481)
expf(1482)
expf(1483)
expf(1484)
expf(1485)
expf(1486)
expf(1487)
expf(1488)
expf(1489)
expf(1490)
expf(1491)
expf(1492)
expf(1493)
expf(1494)
expf(1495)
expf(1496)
expf(1497)
expf(1498)
expf(1499)
expf(1500)
expf(1501)
expf(1502)
expf(1503)
expf(1504)
expf(1505)
expf(1506)
expf(1507)
expf(1508)
expf(1509)
expf(1510)
expf(1511)
expf(1512)
expf(1513)
expf(1514)
expf(1515)
expf(1516)
expf(1517)
expf(1518)
expf(1519)
expf(1520)
expf(1521)
expf(1522)
expf(1523)
expf(1524)
expf(1525)
expf(1526)
expf(1527)
expf(1528)
expf(1529)
expf(1530)
expf(1531)
expf(1532)
expf(1533)
expf(1534)
expf(1535)
expf(1536)
expf(1537)
expf(1538)
expf(1539)
expf(1540)
expf(1541)
expf(1542)
expf(1543)
expf(1544)
expf(1545)
expf(1546)
expf(1547)
expf(1548)
expf(1549)
expf(1550)
expf(1551)
expf(1552)
expf(1553)
expf(1554)
expf(1555)
expf(1556)
expf(1557)
expf(1558)
expf(1559)
expf(1560)
expf(1561)
expf(1562)
expf(1563)
expf(1564)
expf(1565)
expf(1566)
expf(1567)
expf(1568)
expf(1569)
expf(1570)
expf(1571)
expf(1572)
expf(1573)
expf(1574)
expf(1575)
expf(1576)
expf(1577)
expf(1578)
expf(1579)
expf(1580)
expf(1581)
expf(1582)
expf(1583)
expf(1584)
expf(1585)
expf(1586)
expf(1587)
expf(1588)
expf(1589)
expf(1590)
expf(1591)
expf(1592)
expf(1593)
expf(1594)
expf(1595)
expf(1596)
expf(1597)
expf(1598)
expf(1599)
expf(1600)
expf(1601)
expf(1602)
expf(1603)
expf(1604)
expf(1605)
expf(1606)
expf(1607)
expf(1608)
expf(1609)
expf(1610)
expf(1611)
expf(1612)
expf(1613)
expf(1614)
expf(1615)
expf(1616)
expf(1617)
expf(1618)
expf(1619)
expf(1620)
expf(1621)
expf(1622)
expf(1623)
expf(1624)
expf(1625)
expf(1626)
expf(1627)
expf(1628)
expf(1629)
expf(1630)
expf(1631)
expf(1632)
expf(1633)
expf(1634)
expf(1635)
expf(1636)
expf(1637)
expf(1638)
expf(1639)
expf(1640)
expf(1641)
expf(1642)
expf(1643)
expf(1644)
expf(1645)
expf(1646)
expf(1647)
expf(1648)
expf(1649)
expf(1650)
expf(1651)
expf(1652)
expf(1653)
expf(1654)
expf(1655)
expf(1656)
expf(1657)
expf(1658)
expf(1659)
expf(1660)
expf(1661)
expf(1662)
expf(1663)
expf(1664)
expf(1665)
expf(1666)
expf(1667)
expf(1668)
expf(1669)
expf(1670)
expf(1671)
expf(1672)
expf(1673)
expf(1674)
expf(1675)
expf(1676)
expf(1677)
expf(1678)
expf(1679)
expf(1680)
expf(1681)
expf(1682)
expf(1683)
expf(1684)
expf(1685)
expf(1686)
expf(1687)
expf(1688)
expf(1689)
expf(1690)
expf(1691)
expf(1692)
expf(1693)
expf(1694)
expf(1695)
expf(1696)
expf(1697)
expf(1698)
expf(1699)
expf(1700)
expf(1701)
expf(1702)
expf(1703)
expf(1704)
expf(1705)
expf(1706)
expf(1707)
expf(1708)
expf(1709)
expf(1710)
expf(1711)
expf(1712)
expf(1713)
expf(1714)
expf(1715)
expf(1716)
expf(1717)
expf(1718)
expf(1719)
expf(1720)
expf(1721)
expf(1722)
expf(1723)
expf(1724)
expf(1725)
expf(1726)
expf(1727)
expf(1728)
expf(1729)
expf(1730)
expf(1731)
expf(1732)
expf(1733)
expf(1734)
expf(1735)
expf(1736)
expf(1737)
expf(1738)
expf(1739)
expf(1740)
expf(1741)
expf(1742)
expf(1743)
expf(1744)
expf(1745)
expf(1746)
expf(1747)
expf(1748)
expf(1749)
expf(1750)
expf(1751)
expf(1752)
expf(1753)
expf(1754)
expf(1755)
expf(1756)
expf(1757)
expf(1758)
expf(1759)
expf(1760)
expf(1761)
expf(1762)
expf(1763)
expf(1764)
expf(1765)
expf(1766)
expf(1767)
expf(1768)
expf(1769)
expf(1770)
expf(1771)
expf(1772)
expf(1773)
expf(1774)
expf(1775)
expf(1776)
expf(1777)
expf(1778)
expf(1779)
expf(1780)
expf(1781)
expf(1782)
expf(1783)
expf(1784)
expf(1785)
expf(1786)
expf(1787)
expf(1788)
expf(1789)
expf(1790)
expf(1791)
expf(1792)
expf(1793)
expf(1794)
expf(1795)
expf(1796)
expf(1797)
expf(1798)
expf(1799)
expf(1800)
expf(1801)
expf(1802)
expf(1803)
expf(1804)
expf(1805)
expf(1806)
expf(1807)
expf(1808)
expf(1809)
expf(1810)
expf(1811)
expf(1812)
expf(1813)
expf(1814)
expf(1815)
expf(1816)
expf(1817)
expf(1818)
expf(1819)
expf(1820)
expf(1821)
expf(1822)
expf(1823)
expf(1824)
expf(1825)
expf(1826)
expf(1827)
expf(1828)
expf(1829)
expf(1830)
expf(1831)
expf(1832)
expf(1833)
expf(1834)
expf(1835)
expf(1836)
expf(1837)
expf(1838)
expf(1839)
expf(1840)
expf(1841)
expf(1842)
expf(1843)
expf(1844)
expf(1845)
expf(1846)
expf(1847)
expf(1848)
expf(1849)
expf(1850)
expf(1851)
expf(1852)
expf(1853)
expf(1854)
expf(1855)
expf(1856)
expf(1857)
expf(1858)
expf(1859)
expf(1860)
expf(1861)
expf(1862)
expf(1863)
expf(1864)
expf(1865)
expf(1866)
expf(1867)
expf(1868)
expf(1869)
expf(1870)
expf(1871)
expf(1872)
expf(1873)
expf(1874)
expf(1875)
expf(1876)
expf(1877)
expf(1878)
expf(1879)
expf(1880)
expf(1881)
expf(1882)
expf(1883)
expf(1884)
expf(1885)
expf(1886)
expf(1887)
expf(1888)
expf(1889)
expf(1890)
expf(1891)
expf(1892)
expf(1893)
expf(1894)
expf(1895)
expf(1896)
expf(1897)
expf(1898)
expf(1899)
expf(1900)
expf(1901)
expf(1902)
expf(1903)
expf(1904)
expf(1905)
expf(1906)
expf(1907)
expf(1908)
expf(1909)
expf(1910)
expf(1911)
expf(1912)
expf(1913)
expf(1914)
expf(1915)
expf(1916)
expf(1917)
expf(1918)
expf(1919)
expf(1920)
expf(1921)
expf(1922)
expf(1923)
expf(1924)
expf(1925)
expf(1926)
expf(1927)
expf(1928)
expf(1929)
expf(1930)
expf(1931)
expf(1932)
expf(1933)
expf(1934)
expf(1935)
expf(1936)
expf(1937)
expf(1938)
expf(1939)
expf(1940)
expf(1941)
expf(1942)
expf(1943)
expf(1944)
expf(1945)
expf(1946)
expf(1947)
expf(1948)
expf(1949)
expf(1950)
expf(1951)
expf(1952)
expf(1953)
expf(1954)
expf(1955)
expf(1956)
expf(1957)
expf(1958)
expf(1959)
expf(1960)
expf(1961)
expf(1962)
expf(1963)
expf(1964)
expf(1965)
expf(1966)
expf(1967)
expf(1968)
expf(1969)
expf(1970)
expf(1971)
expf(1972)
expf(1973)
expf(1974)
expf(1975)
expf(1976)
expf(1977)
expf(1978)
expf(1979)
expf(1980)
expf(1981)
expf(1982)
expf(1983)
expf(1984)
expf(1985)
expf(1986)
expf(1987)
expf(1988)
expf(1989)
expf(1990)
expf(1991)
expf(1992)
expf(1993)
expf(1994)
expf(1995)
expf(1996)
expf(1997)
expf(1998)
expf(1999)
expf(2000)
expf(2001)
expf(2002)
expf(2003)
expf(2004)
expf(2005)
expf(2006)
expf(2007)
expf(2008)
expf(2009)
expf(2010)
expf(2011)
expf(2012)
expf(2013)
expf(2014)
expf(2015)
expf(2016)
expf(2017)
expf(2018)
expf(2019)
expf(2020)
expf(2021)
expf(2022)
expf(2023)
expf(2024)
expf(2025)
expf(2026)
expf(2027)
expf(2028)
expf(2029)
expf(2030)
expf(2031)
expf(2032)
expf(2033)
expf(2034)
expf(2035)
expf(2036)
expf(2037)
expf(2038)
expf(2039)
expf(2040)
expf(2041)
expf(2042)
expf(2043)
expf(2044)
expf(2045)
expf(2046)
expf(2047)
expf(2048)
expf(2049)
expf(2050)
expf(2051)
expf(2052)
expf(2053)
expf(2054)
expf(2055)
expf(2056)
expf(2057)
expf(2058)
expf(2059)
expf(2060)
expf(2061)
expf(2062)
expf(2063)
expf(2064)
expf(2065)
expf(2066)
expf(2067)
expf(2068)
expf(2069)
expf(2070)
expf(2071)
expf(2072)
expf(2073)
expf(2074)
expf(2075)
expf(2076)
expf(2077)
expf(2078)
expf(2079)
expf(2080)
expf(2081)
expf(2082)
expf(2083)
expf(2084)
expf(2085)
expf(2086)
expf(2087)
expf(2088)
expf(2089)
expf(2090)
expf(2091)
expf(2092)
expf(2093)
expf(2094)
expf(2095)
expf(2096)
expf(2097)
expf(2098)
expf(2099)
expf(2100)
expf(2101)
expf(2102)
expf(2103)
expf(2104)
expf(2105)
expf(2106)
expf(2107)
expf(2108)
expf(2109)
expf(2110)
expf(2111)
expf(2112)
expf(2113)
expf(2114)
expf(2115)
expf(2116)
expf(2117)
expf(2118)
expf(2119)
expf(2120)
expf(2121)
expf(2122)
expf(2123)
expf(2124)
expf(2125)
expf(2126)
expf(2127)
expf(2128)
expf(2129)
expf(2130)
expf(2131)
expf(2132)
expf(2133)
expf(2134)
expf(2135)
expf(2136)
expf(2137)
expf(2138)
expf(2139)
expf(2140)
expf(2141)
expf(2142)
expf(2143)
expf(2144)
expf(2145)
expf(2146)
expf(2147)
expf(2148)
expf(2149)
expf(2150)
expf(2151)
expf(2152)
expf(2153)
expf(2154)
expf(2155)
expf(2156)
expf(2157)
expf(2158)
expf(2159)
expf(2160)
expf(2161)
expf(2162)
expf(2163)
expf(2164)
expf(2165)
expf(2166)
expf(2167)
expf(2168)
expf(2169)
expf(2170)
expf(2171)
expf(2172)
expf(2173)
expf(2174)
expf(2175)
expf(2176)
expf(2177)
expf(2178)
expf(2179)
expf(2180)
expf(2181)
expf(2182)
expf(2183)
expf(2184)
expf(2185)
expf(2186)
expf(2187)
expf(2188)
expf(2189)
expf(2190)
expf(2191)
expf(2192)
expf(2193)
expf(2194)
expf(2195)
expf(2196)
expf(2197)
expf(2198)
expf(2199)
expf(2200)
expf(2201)
expf(2202)
expf(2203)
expf(2204)
expf(2205)
expf(2206)
expf(2207)
expf(2208)
expf(2209)
expf(2210)
expf(2211)
expf(2212)
expf(2213)
expf(2214)
expf(2215)
expf(2216)
expf(2217)
expf(2218)
expf(2219)
expf(2220)
expf(2221)
expf(2222)
expf(2223)
expf(2224)
expf(2225)
expf(2226)
expf(2227)
expf(2228)
expf(2229)
expf(2230)
expf(2231)
expf(2232)
expf(2233)
expf(2234)
expf(2235)
expf(2236)
expf(2237)
expf(2238)
expf(2239)
expf(2240)
expf(2241)
expf(2242)
expf(2243)
expf(2244)
expf(2245)
expf(2246)
expf(2247)
expf(2248)
expf(2249)
expf(2250)
expf(2251)
expf(2252)
expf(2253)
expf(2254)
expf(2255)
expf(2256)
expf(2257)
expf(2258)
expf(2259)
expf(2260)
expf(2261)
expf(2262)
expf(2263)
expf(2264)
expf(2265)
expf(2266)
expf(2267)
expf(2268)
expf(2269)
expf(2270)
expf(2271)
expf(2272)
expf(2273)
expf(2274)
expf(2275)
expf(2276)
expf(2277)
expf(2278)
expf(2279)
expf(2280)
expf(2281)
expf(2282)
expf(2283)
expf(2284)
expf(2285)
expf(2286)
expf(2287)
expf(2288)
expf(2289)
expf(2290)
expf(2291)
expf(2292)
expf(2293)
expf(2294)
expf(2295)
expf(2296)
expf(2297)
expf(2298)
expf(2299)
expf(2300)
expf(2301)
expf(2302)
expf(2303)
expf(2304)
expf(2305)
expf(2306)
expf(2307)
expf(2308)
expf(2309)
expf(2310)
expf(2311)
expf(2312)
expf(2313)
expf(2314)
expf(2315)
expf(2316)
expf(2317)
expf(2318)
expf(2319)
expf(2320)
expf(2321)
expf(2322)
expf(2323)
expf(2324)
expf(2325)
expf(2326)
expf(2327)
expf(2328)
expf(2329)
expf(2330)
expf(2331)
expf(2332)
expf(2333)
expf(2334)
expf(2335)
expf(2336)
expf(2337)
expf(2338)
expf(2339)
expf(2340)
expf(2341)
expf(2342)
expf(2343)
expf(2344)
expf(2345)
expf(2346)
expf(2347)
expf(2348)
expf(2349)
expf(2350)
expf(2351)
expf(2352)
expf(2353)
expf(2354)
expf(2355)
expf(2356)
expf(2357)
expf(2358)
expf(2359)
expf(2360)
expf(2361)
expf(2362)
expf(2363)
expf(2364)
expf(2365)
expf(2366)
expf(2367)
expf(2368)
expf(2369)
expf(2370)
expf(2371)
expf(2372)
expf(2373)
expf(2374)
expf(2375)
expf(2376)
expf(2377)
expf(2378)
expf(2379)
expf(2380)
expf(2381)
expf(2382)
expf(2383)
expf(2384)
expf(2385)
expf(2386)
expf(2387)
expf(2388)
expf(2389)
expf(2390)
expf(2391)
expf(2392)
expf(2393)
expf(2394)
expf(2395)
expf(2396)
expf(2397)
expf(2398)
expf(2399)
expf(2400)
expf(2401)
expf(2402)
expf(2403)
expf(2404)
expf(2405)
expf(2406)
expf(2407)
expf(2408)
expf(2409)
expf(2410)
expf(2411)
expf(2412)
expf(2413)
expf(2414)
expf(2415)
expf(2416)
expf(2417)
expf(2418)
expf(2419)
expf(2420)
expf(2421)
expf(2422)
expf(2423)
expf(2424)
expf(2425)
expf(2426)
expf(2427)
expf(2428)
expf(2429)
expf(2430)
expf(2431)
expf(2432)
expf(2433)
expf(2434)
expf(2435)
expf(2436)
expf(2437)
expf(2438)
expf(2439)
expf(2440)
expf(2441)
expf(2442)
expf(2443)
expf(2444)
expf(2445)
expf(2446)
expf(2447)
expf(2448)
expf(2449)
expf(2450)
expf(2451)
expf(2452)
expf(2453)
expf(2454)
expf(2455)
expf(2456)
expf(2457)
expf(2458)
expf(2459)
expf(2460)
expf(2461)
expf(2462)
expf(2463)
expf(2464)
expf(2465)
expf(2466)
expf(2467)
expf(2468)
expf(2469)
expf(2470)
expf(2471)
expf(2472)
expf(2473)
expf(2474)
expf(2475)
expf(2476)
expf(2477)
expf(2478)
expf(2479)
expf(2480)
expf(2481)
expf(2482)
expf(2483)
expf(2484)
expf(2485)
expf(2486)
expf(2487)
expf(2488)
expf(2489)
expf(2490)
expf(2491)
expf(2492)
expf(2493)
expf(2494)
expf(2495)
expf(2496)
expf(2497)
expf(2498)
expf(2499)
expf(2500)
expf(2501)
expf(2502)
expf(2503)
expf(2504)
expf(2505)
expf(2506)
expf(2507)
expf(2508)
expf(2509)
expf(2510)
expf(2511)
expf(2512)
expf(2513)
expf(2514)
expf(2515)
expf(2516)
expf(2517)
expf(2518)
expf(2519)
expf(2520)
expf(2521)
expf(2522)
expf(2523)
expf(2524)
expf(2525)
expf(2526)
expf(2527)
expf(2528)
expf(2529)
expf(2530)
expf(2531)
expf(2532)
expf(2533)
expf(2534)
expf(2535)
expf(2536)
expf(2537)
expf(2538)
expf(2539)
expf(2540)
expf(2541)
expf(2542)
expf(2543)
expf(2544)
expf(2545)
expf(2546)
expf(2547)
expf(2548)
expf(2549)
expf(2550)
expf(2551)
expf(2552)
expf(2553)
expf(2554)
expf(2555)
expf(2556)
expf(2557)
expf(2558)
expf(2559)
expf(2560)
expf(2561)
expf(2562)
expf(2563)
expf(2564)
expf(2565)
expf(2566)
expf(2567)
expf(2568)
expf(2569)
expf(2570)
expf(2571)
expf(2572)
expf(2573)
expf(2574)
expf(2575)
expf(2576)
expf(2577)
expf(2578)
expf(2579)
expf(2580)
expf(2581)
expf(2582)
expf(2583)
expf(2584)
expf(2585)
expf(2586)
expf(2587)
expf(2588)
expf(2589)
expf(2590)
expf(2591)
expf(2592)
expf(2593)
expf(2594)
expf(2595)
expf(2596)
expf(2597)
expf(2598)
expf(2599)
expf(2600)
expf(2601)
expf(2602)
expf(2603)
expf(2604)
expf(2605)
expf(2606)
expf(2607)
expf(2608)
expf(2609)
expf(2610)
expf(2611)
expf(2612)
expf(2613)
expf(2614)
expf(2615)
expf(2616)
expf(2617)
expf(2618)
expf(2619)
expf(2620)
expf(2621)
expf(2622)
expf(2623)
expf(2624)
expf(2625)
expf(2626)
expf(2627)
expf(2628)
expf(2629)
expf(2630)
expf(2631)
expf(2632)
expf(2633)
expf(2634)
expf(2635)
expf(2636)
expf(2637)
expf(2638)
expf(2639)
expf(2640)
expf(2641)
expf(2642)
expf(2643)
expf(2644)
expf(2645)
expf(2646)
expf(2647)
expf(2648)
expf(2649)
expf(2650)
expf(2651)
expf(2652)
expf(2653)
expf(2654)
expf(2655)
expf(2656)
expf(2657)
expf(2658)
expf(2659)
expf(2660)
expf(2661)
expf(2662)
expf(2663)
expf(2664)
expf(2665)
expf(2666)
expf(2667)
expf(2668)
expf(2669)
expf(2670)
expf(2671)
expf(2672)
expf(2673)
expf(2674)
expf(2675)
expf(2676)
expf(2677)
expf(2678)
expf(2679)
expf(2680)
expf(2681)
expf(2682)
expf(2683)
expf(2684)
expf(2685)
expf(2686)
expf(2687)
expf(2688)
expf(2689)
expf(2690)
expf(2691)
expf(2692)
expf(2693)
expf(2694)
expf(2695)
expf(2696)
expf(2697)
expf(2698)
expf(2699)
expf(2700)
expf(2701)
expf(2702)
expf(2703)
expf(2704)
expf(2705)
expf(2706)
expf(2707)
expf(2708)
expf(2709)
expf(2710)
expf(2711)
expf(2712)
expf(2713)
expf(2714)
expf(2715)
expf(2716)
expf(2717)
expf(2718)
expf(2719)
expf(2720)
expf(2721)
expf(2722)
expf(2723)
expf(2724)
expf(2725)
expf(2726)
expf(2727)
expf(2728)
expf(2729)
expf(2730)
expf(2731)
expf(2732)
expf(2733)
expf(2734)
expf(2735)
expf(2736)
expf(2737)
expf(2738)
expf(2739)
expf(2740)
expf(2741)
expf(2742)
expf(2743)
expf(2744)
expf(2745)
expf(2746)
expf(2747)
expf(2748)
expf(2749)
expf(2750)
expf(2751)
expf(2752)
expf(2753)
expf(2754)
expf(2755)
expf(2756)
expf(2757)
expf(2758)
expf(2759)
expf(2760)
expf(2761)
expf(2762)
expf(2763)
expf(2764)
expf(2765)
expf(2766)
expf(2767)
expf(2768)
expf(2769)
expf(2770)
expf(2771)
expf(2772)
expf(2773)
expf(2774)
expf(2775)
expf(2776)
expf(2777)
expf(2778)
expf(2779)
expf(2780)
expf(2781)
expf(2782)
expf(2783)
expf(2784)
expf(2785)
expf(2786)
expf(2787)
expf(2788)
expf(2789)
expf(2790)
expf(2791)
expf(2792)
expf(2793)
expf(2794)
expf(2795)
expf(2796)
expf(2797)
expf(2798)
expf(2799)
expf(2800)
expf(2801)
expf(2802)
expf(2803)
expf(2804)
expf(2805)
expf(2806)
expf(2807)
expf(2808)
expf(2809)
expf(2810)
expf(2811)
expf(2812)
expf(2813)
expf(2814)
expf(2815)
expf(2816)
expf(2817)
expf(2818)
expf(2819)
expf(2820)
expf(2821)
expf(2822)
expf(2823)
expf(2824)
expf(2825)
expf(2826)
expf(2827)
expf(2828)
expf(2829)
expf(2830)
expf(2831)
expf(2832)
expf(2833)
expf(2834)
expf(2835)
expf(2836)
expf(2837)
expf(2838)
expf(2839)
expf(2840)
expf(2841)
expf(2842)
expf(2843)
expf(2844)
expf(2845)
expf(2846)
expf(2847)
expf(2848)
expf(2849)
expf(2850)
expf(2851)
expf(2852)
expf(2853)
expf(2854)
expf(2855)
expf(2856)
expf(2857)
expf(2858)
expf(2859)
expf(2860)
expf(2861)
expf(2862)
expf(2863)
expf(2864)
expf(2865)
expf(2866)
expf(2867)
expf(2868)
expf(2869)
expf(2870)
expf(2871)
expf(2872)
expf(2873)
expf(2874)
expf(2875)
expf(2876)
expf(2877)
expf(2878)
expf(2879)
expf(2880)
expf(2881)
expf(2882)
expf(2883)
expf(2884)
expf(2885)
expf(2886)
expf(2887)
expf(2888)
expf(2889)
expf(2890)
expf(2891)
expf(2892)
expf(2893)
expf(2894)
expf(2895)
expf(2896)
expf(2897)
expf(2898)
expf(2899)
expf(2900)
expf(2901)
expf(2902)
expf(2903)
expf(2904)
expf(2905)
expf(2906)
expf(2907)
expf(2908)
expf(2909)
expf(2910)
expf(2911)
expf(2912)
expf(2913)
expf(2914)
expf(2915)
expf(2916)
expf(2917)
expf(2918)
expf(2919)
expf(2920)
expf(2921)
expf(2922)
expf(2923)
expf(2924)
expf(2925)
expf(2926)
expf(2927)
expf(2928)
expf(2929)
expf(2930)
expf(2931)
expf(2932)
expf(2933)
expf(2934)
expf(2935)
expf(2936)
expf(2937)
expf(2938)
expf(2939)
expf(2940)
expf(2941)
expf(2942)
expf(2943)
expf(2944)
expf(2945)
expf(2946)
expf(2947)
expf(2948)
expf(2949)
expf(2950)
expf(2951)
expf(2952)
expf(2953)
expf(2954)
expf(2955)
expf(2956)
expf(2957)
expf(2958)
expf(2959)
expf(2960)
expf(2961)
expf(2962)
expf(2963)
expf(2964)
expf(2965)
expf(2966)
expf(2967)
expf(2968)
expf(2969)
expf(2970)
expf(2971)
expf(2972)
expf(2973)
expf(2974)
expf(2975)
expf(2976)
expf(2977)
expf(2978)
expf(2979)
expf(2980)
expf(2981)
expf(2982)
expf(2983)
expf(2984)
expf(2985)
expf(2986)
expf(2987)
expf(2988)
expf(2989)
expf(2990)
expf(2991)
expf(2992)
expf(2993)
expf(2994)
expf(2995)
expf(2996)
expf(2997)
expf(2998)
expf(2999)
expf(3000)
expf(3001)
expf(3002)
expf(3003)
expf(3004)
expf(3005)
expf(3006)
expf(3007)
expf(3008)
expf(3009)
expf(3010)
expf(3011)
expf(3012)
expf(3013)
expf(3014)
expf(3015)
expf(3016)
expf(3017)
expf(3018)
expf(3019)
expf(3020)
expf(3021)
expf(3022)
expf(3023)
expf(3024)
expf(3025)
expf(3026)
expf(3027)
expf(3028)
expf(3029)
expf(3030)
expf(3031)
expf(3032)
expf(3033)
expf(3034)
expf(3035)
expf(3036)
expf(3037)
expf(3038)
expf(3039)
expf(3040)
expf(3041)
expf(3042)
expf(3043)
expf(3044)
expf(3045)
expf(3046)
expf(3047)
expf(3048)
expf(3049)
expf(3050)
expf(3051)
expf(3052)
expf(3053)
expf(3054)
expf(3055)
expf(3056)
expf(3057)
expf(3058)
expf(3059)
expf(3060)
expf(3061)
expf(3062)
expf(3063)
expf(3064)
expf(3065)
expf(3066)
expf(3067)
expf(3068)
expf(3069)
expf(3070)
expf(3071)
expf(3072)
expf(3073)
expf(3074)
expf(3075)
expf(3076)
expf(3077)
expf(3078)
expf(3079)
expf(3080)
expf(3081)
expf(3082)
expf(3083)
expf(3084)
expf(3085)
expf(3086)
expf(3087)
expf(3088)
expf(3089)
expf(3090)
expf(3091)
expf(3092)
expf(3093)
expf(3094)
expf(3095)
expf(3096)
expf(3097)
expf(3098)
expf(3099)
expf(3100)
expf(3101)
expf(3102)
expf(3103)
expf(3104)
expf(3105)
expf(3106)
expf(3107)
expf(3108)
expf(3109)
expf(3110)
expf(3111)
expf(3112)
expf(3113)
expf(3114)
expf(3115)
expf(3116)
expf(3117)
expf(3118)
expf(3119)
expf(3120)
expf(3121)
expf(3122)
expf(3123)
expf(3124)
expf(3125)
expf(3126)
expf(3127)
expf(3128)
expf(3129)
expf(3130)
expf(3131)
expf(3132)
expf(3133)
expf(3134)
expf(3135)
expf(3136)
expf(3137)
expf(3138)
expf(3139)
expf(3140)
expf(3141)
expf(3142)
expf(3143)
expf(3144)
expf(3145)
expf(3146)
expf(3147)
expf(3148)
expf(3149)
expf(3150)
expf(3151)
expf(3152)
expf(3153)
expf(3154)
expf(3155)
expf(3156)
expf(3157)
expf(3158)
expf(3159)
expf(3160)
expf(3161)
expf(3162)
expf(3163)
expf(3164)
expf(3165)
expf(3166)
expf(3167)
expf(3168)
expf(3169)
expf(3170)
expf(3171)
expf(3172)
expf(3173)
expf(3174)
expf(3175)
expf(3176)
expf(3177)
expf(3178)
expf(3179)
expf(3180)
expf(3181)
expf(3182)
expf(3183)
expf(3184)
expf(3185)
expf(3186)
expf(3187)
expf(3188)
expf(3189)
expf(3190)
expf(3191)
expf(3192)
expf(3193)
expf(3194)
expf(3195)
expf(3196)
expf(3197)
expf(3198)
expf(3199)
expf(3200)
expf(3201)
expf(3202)
expf(3203)
expf(3204)
expf(3205)
expf(3206)
expf(3207)
expf(3208)
expf(3209)
expf(3210)
expf(3211)
expf(3212)
expf(3213)
expf(3214)
expf(3215)
expf(3216)
expf(3217)
expf(3218)
expf(3219)
expf(3220)
expf(3221)
expf(3222)
expf(3223)
expf(3224)
expf(3225)
expf(3226)
expf(3227)
expf(3228)
expf(3229)
expf(3230)
expf(3231)
expf(3232)
expf(3233)
expf(3234)
expf(3235)
expf(3236)
expf(3237)
expf(3238)
expf(3239)
expf(3240)
expf(3241)
expf(3242)
expf(3243)
expf(3244)
expf(3245)
expf(3246)
expf(3247)
expf(3248)
expf(3249)
expf(3250)
expf(3251)
expf(3252)
expf(3253)
expf(3254)
expf(3255)
expf(3256)
expf(3257)
expf(3258)
expf(3259)
expf(3260)
expf(3261)
expf(3262)
expf(3263)
expf(3264)
expf(3265)
expf(3266)
expf(3267)
expf(3268)
expf(3269)
expf(3270)
expf(3271)
expf(3272)
expf(3273)
expf(3274)
expf(3275)
expf(3276)
expf(3277)
expf(3278)
expf(3279)
expf(3280)
expf(3281)
expf(3282)
expf(3283)
expf(3284)
expf(3285)
expf(3286)
expf(3287)
expf(3288)
expf(3289)
expf(3290)
expf(3291)
expf(3292)
expf(3293)
expf(3294)
expf(3295)
expf(3296)
expf(3297)
expf(3298)
expf(3299)
expf(3300)
expf(3301)
expf(3302)
expf(3303)
expf(3304)
expf(3305)
expf(3306)
expf(3307)
expf(3308)
expf(3309)
expf(3310)
expf(3311)
expf(3312)
expf(3313)
expf(3314)
expf(3315)
expf(3316)
expf(3317)
expf(3318)
expf(3319)
expf(3320)
expf(3321)
expf(3322)
expf(3323)
expf(3324)
expf(3325)
expf(3326)
expf(3327)
expf(3328)
expf(3329)
expf(3330)
expf(3331)
expf(3332)
expf(3333)
expf(3334)
expf(3335)
expf(3336)
expf(3337)
expf(3338)
expf(3339)
expf(3340)
expf(3341)
expf(3342)
expf(3343)
expf(3344)
expf(3345)
expf(3346)
expf(3347)
expf(3348)
expf(3349)
expf(3350)
expf(3351)
expf(3352)
expf(3353)
expf(3354)
expf(3355)
expf(3356)
expf(3357)
expf(3358)
expf(3359)
expf(3360)
expf(3361)
expf(3362)
expf(3363)
expf(3364)
expf(3365)
expf(3366)
expf(3367)
expf(3368)
expf(3369)
expf(3370)
expf(3371)
expf(3372)
expf(3373)
expf(3374)
expf(3375)
expf(3376)
expf(3377)
expf(3378)
expf(3379)
expf(3380)
expf(3381)
expf(3382)
expf(3383)
expf(3384)
expf(3385)
expf(3386)
expf(3387)
expf(3388)
expf(3389)
expf(3390)
expf(3391)
expf(3392)
expf(3393)
expf(3394)
expf(3395)
expf(3396)
expf(3397)
expf(3398)
expf(3399)
expf(3400)
expf(3401)
expf(3402)
expf(3403)
expf(3404)
expf(3405)
expf(3406)
expf(3407)
expf(3408)
expf(3409)
expf(3410)
expf(3411)
expf(3412)
expf(3413)
expf(3414)
expf(3415)
expf(3416)
expf(3417)
expf(3418)
expf(3419)
expf(3420)
expf(3421)
expf(3422)
expf(3423)
expf(3424)
expf(3425)
expf(3426)
expf(3427)
expf(3428)
expf(3429)
expf(3430)
expf(3431)
expf(3432)
expf(3433)
expf(3434)
expf(3435)
expf(3436)
expf(3437)
expf(3438)
expf(3439)
expf(3440)
expf(3441)
expf(3442)
expf(3443)
expf(3444)
expf(3445)
expf(3446)
expf(3447)
expf(3448)
expf(3449)
expf(3450)
expf(3451)
expf(3452)
expf(3453)
expf(3454)
expf(3455)
expf(3456)
expf(3457)
expf(3458)
expf(3459)
expf(3460)
expf(3461)
expf(3462)
expf(3463)
expf(3464)
expf(3465)
expf(3466)
expf(3467)
expf(3468)
expf(3469)
expf(3470)
expf(3471)
expf(3472)
expf(3473)
expf(3474)
expf(3475)
expf(3476)
expf(3477)
expf(3478)
expf(3479)
expf(3480)
expf(3481)
expf(3482)
expf(3483)
expf(3484)
expf(3485)
expf(3486)
expf(3487)
expf(3488)
expf(3489)
expf(3490)
expf(3491)
expf(3492)
expf(3493)
expf(3494)
expf(3495)
expf(3496)
expf(3497)
expf(3498)
expf(3499)
expf(3500)
expf(3501)
expf(3502)
expf(3503)
expf(3504)
expf(3505)
expf(3506)
expf(3507)
expf(3508)
expf(3509)
expf(3510)
expf(3511)
expf(3512)
expf(3513)
expf(3514)
expf(3515)
expf(3516)
expf(3517)
expf(3518)
expf(3519)
expf(3520)
expf(3521)
expf(3522)
expf(3523)
expf(3524)
expf(3525)
expf(3526)
expf(3527)
expf(3528)
expf(3529)
expf(3530)
expf(3531)
expf(3532)
expf(3533)
expf(3534)
expf(3535)
expf(3536)
expf(3537)
expf(3538)
expf(3539)
expf(3540)
expf(3541)
expf(3542)
expf(3543)
expf(3544)
expf(3545)
expf(3546)
expf(3547)
expf(3548)
expf(3549)
expf(3550)
expf(3551)
expf(3552)
expf(3553)
expf(3554)
expf(3555)
expf(3556)
expf(3557)
expf(3558)
expf(3559)
expf(3560)
expf(3561)
expf(3562)
expf(3563)
expf(3564)
expf(3565)
expf(3566)
expf(3567)
expf(3568)
expf(3569)
expf(3570)
expf(3571)
expf(3572)
expf(3573)
expf(3574)
expf(3575)
expf(3576)
expf(3577)
expf(3578)
expf(3579)
expf(3580)
expf(3581)
expf(3582)
expf(3583)
expf(3584)
expf(3585)
expf(3586)
expf(3587)
expf(3588)
expf(3589)
expf(3590)
expf(3591)
expf(3592)
expf(3593)
expf(3594)
expf(3595)
expf(3596)
expf(3597)
expf(3598)
expf(3599)
expf(3600)
expf(3601)
expf(3602)
expf(3603)
expf(3604)
expf(3605)
expf(3606)
expf(3607)
expf(3608)
expf(3609)
expf(3610)
expf(3611)
expf(3612)
expf(3613)
expf(3614)
expf(3615)
expf(3616)
expf(3617)
expf(3618)
expf(3619)
expf(3620)
expf(3621)
expf(3622)
expf(3623)
expf(3624)
expf(3625)
expf(3626)
expf(3627)
expf(3628)
expf(3629)
expf(3630)
expf(3631)
expf(3632)
expf(3633)
expf(3634)
expf(3635)
expf(3636)
expf(3637)
expf(3638)
expf(3639)
expf(3640)
expf(3641)
expf(3642)
expf(3643)
expf(3644)
expf(3645)
expf(3646)
expf(3647)
expf(3648)
expf(3649)
expf(3650)
expf(3651)
expf(3652)
expf(3653)
expf(3654)
expf(3655)
expf(3656)
expf(3657)
expf(3658)
expf(3659)
expf(3660)
expf(3661)
expf(3662)
expf(3663)
expf(3664)
expf(3665)
expf(3666)
expf(3667)
expf(3668)
expf(3669)
expf(3670)
expf(3671)
expf(3672)
expf(3673)
expf(3674)
expf(3675)
expf(3676)
expf(3677)
expf(3678)
expf(3679)
expf(3680)
expf(3681)
expf(3682)
expf(3683)
expf(3684)
expf(3685)
expf(3686)
expf(3687)
expf(3688)
expf(3689)
expf(3690)
expf(3691)
expf(3692)
expf(3693)
expf(3694)
expf(3695)
expf(3696)
expf(3697)
expf(3698)
expf(3699)
expf(3700)
expf(3701)
expf(3702)
expf(3703)
expf(3704)
expf(3705)
expf(3706)
expf(3707)
expf(3708)
expf(3709)
expf(3710)
expf(3711)
expf(3712)
expf(3713)
expf(3714)
expf(3715)
expf(3716)
expf(3717)
expf(3718)
expf(3719)
expf(3720)
expf(3721)
expf(3722)
expf(3723)
expf(3724)
expf(3725)
expf(3726)
expf(3727)
expf(3728)
expf(3729)
expf(3730)
expf(3731)
expf(3732)
expf(3733)
expf(3734)
expf(3735)
expf(3736)
expf(3737)
expf(3738)
expf(3739)
expf(3740)
expf(3741)
expf(3742)
expf(3743)
expf(3744)
expf(3745)
expf(3746)
expf(3747)
expf(3748)
expf(3749)
expf(3750)
expf(3751)
expf(3752)
expf(3753)
expf(3754)
expf(3755)
expf(3756)
expf(3757)
expf(3758)
expf(3759)
expf(3760)
expf(3761)
expf(3762)
expf(3763)
expf(3764)
expf(3765)
expf(3766)
expf(3767)
expf(3768)
expf(3769)
expf(3770)
expf(3771)
expf(3772)
expf(3773)
expf(3774)
expf(3775)
expf(3776)
expf(3777)
expf(3778)
expf(3779)
expf(3780)
expf(3781)
expf(3782)
expf(3783)
expf(3784)
expf(3785)
expf(3786)
expf(3787)
expf(3788)
expf(3789)
expf(3790)
expf(3791)
expf(3792)
expf(3793)
expf(3794)
expf(3795)
expf(3796)
expf(3797)
expf(3798)
expf(3799)
expf(3800)
expf(3801)
expf(3802)
expf(3803)
expf(3804)
expf(3805)
expf(3806)
expf(3807)
expf(3808)
expf(3809)
expf(3810)
expf(3811)
expf(3812)
expf(3813)
expf(3814)
expf(3815)
expf(3816)
expf(3817)
expf(3818)
expf(3819)
expf(3820)
expf(3821)
expf(3822)
expf(3823)
expf(3824)
expf(3825)
expf(3826)
expf(3827)
expf(3828)
expf(3829)
expf(3830)
expf(3831)
expf(3832)
expf(3833)
expf(3834)
expf(3835)
expf(3836)
expf(3837)
expf(3838)
expf(3839)
expf(3840)
expf(3841)
expf(3842)
expf(3843)
expf(3844)
expf(3845)
expf(3846)
expf(3847)
expf(3848)
expf(3849)
expf(3850)
expf(3851)
expf(3852)
expf(3853)
expf(3854)
expf(3855)
expf(3856)
expf(3857)
expf(3858)
expf(3859)
expf(3860)
expf(3861)
expf(3862)
expf(3863)
expf(3864)
expf(3865)
expf(3866)
expf(3867)
expf(3868)
expf(3869)
expf(3870)
expf(3871)
expf(3872)
expf(3873)
expf(3874)
expf(3875)
expf(3876)
expf(3877)
expf(3878)
expf(3879)
expf(3880)
expf(3881)
expf(3882)
expf(3883)
expf(3884)
expf(3885)
expf(3886)
expf(3887)
expf(3888)
expf(3889)
expf(3890)
expf(3891)
expf(3892)
expf(3893)
expf(3894)
expf(3895)
expf(3896)
expf(3897)
expf(3898)
expf(3899)
expf(3900)
expf(3901)
expf(3902)
expf(3903)
expf(3904)
expf(3905)
expf(3906)
expf(3907)
expf(3908)
expf(3909)
expf(3910)
expf(3911)
expf(3912)
expf(3913)
expf(3914)
expf(3915)
expf(3916)
expf(3917)
expf(3918)
expf(3919)
expf(3920)
expf(3921)
expf(3922)
expf(3923)
expf(3924)
expf(3925)
expf(3926)
expf(3927)
expf(3928)
expf(3929)
expf(3930)
expf(3931)
expf(3932)
expf(3933)
expf(3934)
expf(3935)
expf(3936)
expf(3937)
expf(3938)
expf(3939)
expf(3940)
expf(3941)
expf(3942)
expf(3943)
expf(3944)
expf(3945)
expf(3946)
expf(3947)
expf(3948)
expf(3949)
expf(3950)
expf(3951)
expf(3952)
expf(3953)
expf(3954)
expf(3955)
expf(3956)
expf(3957)
expf(3958)
expf(3959)
expf(3960)
expf(3961)
expf(3962)
expf(3963)
expf(3964)
expf(3965)
expf(3966)
expf(3967)
expf(3968)
expf(3969)
expf(3970)
expf(3971)
expf(3972)
expf(3973)
expf(3974)
expf(3975)
expf(3976)
expf(3977)
expf(3978)
expf(3979)
expf(3980)
expf(3981)
expf(3982)
expf(3983)
expf(3984)
expf(3985)
expf(3986)
expf(3987)
expf(3988)
expf(3989)
expf(3990)
expf(3991)
expf(3992)
expf(3993)
expf(3994)
expf(3995)
expf(3996)
expf(3997)
expf(3998)
expf(3999)
expf(4000)
expf(4001)
expf(4002)
expf(4003)
expf(4004)
expf(4005)
expf(4006)
expf(4007)
expf(4008)
expf(4009)
expf(4010)
expf(4011)
expf(4012)
expf(4013)
expf(4014)
expf(4015)
expf(4016)
expf(4017)
expf(4018)
expf(4019)
expf(4020)
expf(4021)
expf(4022)
expf(4023)
expf(4024)
expf(4025)
expf(4026)
expf(4027)
expf(4028)
expf(4029)
expf(4030)
expf(4031)
expf(4032)
expf(4033)
expf(4034)
expf(4035)
expf(4036)
expf(4037)
expf(4038)
expf(4039)
expf(4040)
expf(4041)
expf(4042)
expf(4043)
expf(4044)
expf(4045)
expf(4046)
expf(4047)
expf(4048)
expf(4049)
expf(4050)
expf(4051)
expf(4052)
expf(4053)
expf(4054)
expf(4055)
expf(4056)
expf(4057)
expf(4058)
expf(4059)
expf(4060)
expf(4061)
expf(4062)
expf(4063)
expf(4064)
expf(4065)
expf(4066)
expf(4067)
expf(4068)
expf(4069)
expf(4070)
expf(4071)
expf(4072)
expf(4073)
expf(4074)
expf(4075)
expf(4076)
expf(4077)
expf(4078)
expf(4079)
expf(4080)
expf(4081)
expf(4082)
expf(4083)
expf(4084)
expf(4085)
expf(4086)
expf(4087)
expf(4088)
expf(4089)
expf(4090)
expf(4091)
expf(4092)
expf(4093)
expf(4094)
expf(4095)
expf(4096)
expf(4097)
expf(4098)
expf(4099)
expf(4100)
expf(4101)
expf(4102)
expf(4103)
expf(4104)
expf(4105)
expf(4106)
expf(4107)
expf(4108)
expf(4109)
expf(4110)
expf(4111)
expf(4112)
expf(4113)
expf(4114)
expf(4115)
expf(4116)
expf(4117)
expf(4118)
expf(4119)
expf(4120)
expf(4121)
expf(4122)
expf(4123)
expf(4124)
expf(4125)
expf(4126)
expf(4127)
expf(4128)
expf(4129)
expf(4130)
expf(4131)
expf(4132)
expf(4133)
expf(4134)
expf(4135)
expf(4136)
expf(4137)
expf(4138)
expf(4139)
expf(4140)
expf(4141)
expf(4142)
expf(4143)
expf(4144)
expf(4145)
expf(4146)
expf(4147)
expf(4148)
expf(4149)
expf(4150)
expf(4151)
expf(4152)
expf(4153)
expf(4154)
expf(4155)
expf(4156)
expf(4157)
expf(4158)
expf(4159)
expf(4160)
expf(4161)
expf(4162)
expf(4163)
expf(4164)
expf(4165)
expf(4166)
expf(4167)
expf(4168)
expf(4169)
expf(4170)
expf(4171)
expf(4172)
expf(4173)
expf(4174)
expf(4175)
expf(4176)
expf(4177)
expf(4178)
expf(4179)
expf(4180)
expf(4181)
expf(4182)
expf(4183)
expf(4184)
expf(4185)
expf(4186)
expf(4187)
expf(4188)
expf(4189)
expf(4190)
expf(4191)
expf(4192)
expf(4193)
expf(4194)
expf(4195)
expf(4196)
expf(4197)
expf(4198)
expf(4199)
expf(4200)
expf(4201)
expf(4202)
expf(4203)
expf(4204)
expf(4205)
expf(4206)
expf(4207)
expf(4208)
expf(4209)
expf(4210)
expf(4211)
expf(4212)
expf(4213)
expf(4214)
expf(4215)
expf(4216)
expf(4217)
expf(4218)
expf(4219)
expf(4220)
expf(4221)
expf(4222)
expf(4223)
expf(4224)
expf(4225)
expf(4226)
expf(4227)
expf(4228)
expf(4229)
expf(4230)
expf(4231)
expf(4232)
expf(4233)
expf(4234)
expf(4235)
expf(4236)
expf(4237)
expf(4238)
expf(4239)
expf(4240)
expf(4241)
expf(4242)
expf(4243)
expf(4244)
expf(4245)
expf(4246)
expf(4247)
expf(4248)
expf(4249)
expf(4250)
expf(4251)
expf(4252)
expf(4253)
expf(4254)
expf(4255)
expf(4256)
expf(4257)
expf(4258)
expf(4259)
expf(4260)
expf(4261)
expf(4262)
expf(4263)
expf(4264)
expf(4265)
expf(4266)
expf(4267)
expf(4268)
expf(4269)
expf(4270)
expf(4271)
expf(4272)
expf(4273)
expf(4274)
expf(4275)
expf(4276)
expf(4277)
expf(4278)
expf(4279)
expf(4280)
expf(4281)
expf(4282)
expf(4283)
expf(4284)
expf(4285)
expf(4286)
expf(4287)
expf(4288)
expf(4289)
expf(4290)
expf(4291)
expf(4292)
expf(4293)
expf(4294)
expf(4295)
expf(4296)
expf(4297)
expf(4298)
expf(4299)
expf(4300)
expf(4301)
expf(4302)
expf(4303)
expf(4304)
expf(4305)
expf(4306)
expf(4307)
expf(4308)
expf(4309)
expf(4310)
expf(4311)
expf(4312)
expf(4313)
expf(4314)
expf(4315)
expf(4316)
expf(4317)
expf(4318)
expf(4319)
expf(4320)
expf(4321)
expf(4322)
expf(4323)
expf(4324)
expf(4325)
expf(4326)
expf(4327)
expf(4328)
expf(4329)
expf(4330)
expf(4331)
expf(4332)
expf(4333)
expf(4334)
expf(4335)
expf(4336)
expf(4337)
expf(4338)
expf(4339)
expf(4340)
expf(4341)
expf(4342)
expf(4343)
expf(4344)
expf(4345)
expf(4346)
expf(4347)
expf(4348)
expf(4349)
expf(4350)
expf(4351)
expf(4352)
expf(4353)
expf(4354)
expf(4355)
expf(4356)
expf(4357)
expf(4358)
expf(4359)
expf(4360)
expf(4361)
expf(4362)
expf(4363)
expf(4364)
expf(4365)
expf(4366)
expf(4367)
expf(4368)
expf(4369)
expf(4370)
expf(4371)
expf(4372)
expf(4373)
expf(4374)
expf(4375)
expf(4376)
expf(4377)
expf(4378)
expf(4379)
expf(4380)
expf(4381)
expf(4382)
expf(4383)
expf(4384)
expf(4385)
expf(4386)
expf(4387)
expf(4388)
expf(4389)
expf(4390)
expf(4391)
expf(4392)
expf(4393)
expf(4394)
expf(4395)
expf(4396)
expf(4397)
expf(4398)
expf(4399)
expf(4400)
expf(4401)
expf(4402)
expf(4403)
expf(4404)
expf(4405)
expf(4406)
expf(4407)
expf(4408)
expf(4409)
expf(4410)
expf(4411)
expf(4412)
expf(4413)
expf(4414)
expf(4415)
expf(4416)
expf(4417)
expf(4418)
expf(4419)
expf(4420)
expf(4421)
expf(4422)
expf(4423)
expf(4424)
expf(4425)
expf(4426)
expf(4427)
expf(4428)
expf(4429)
expf(4430)
expf(4431)
expf(4432)
expf(4433)
expf(4434)
expf(4435)
expf(4436)
expf(4437)
expf(4438)
expf(4439)
expf(4440)
expf(4441)
expf(4442)
expf(4443)
expf(4444)
expf(4445)
expf(4446)
expf(4447)
expf(4448)
expf(4449)
expf(4450)
expf(4451)
expf(4452)
expf(4453)
expf(4454)
expf(4455)
expf(4456)
expf(4457)
expf(4458)
expf(4459)
expf(4460)
expf(4461)
expf(4462)
expf(4463)
expf(4464)
expf(4465)
expf(4466)
expf(4467)
expf(4468)
expf(4469)
expf(4470)
expf(4471)
expf(4472)
expf(4473)
expf(4474)
expf(4475)
expf(4476)
expf(4477)
expf(4478)
expf(4479)
expf(4480)
expf(4481)
expf(4482)
expf(4483)
expf(4484)
expf(4485)
expf(4486)
expf(4487)
expf(4488)
expf(4489)
expf(4490)
expf(4491)
expf(4492)
expf(4493)
expf(4494)
expf(4495)
expf(4496)
expf(4497)
expf(4498)
expf(4499)
expf(4500)
expf(4501)
expf(4502)
expf(4503)
expf(4504)
expf(4505)
expf(4506)
expf(4507)
expf(4508)
expf(4509)
expf(4510)
expf(4511)
expf(4512)
expf(4513)
expf(4514)
expf(4515)
expf(4516)
expf(4517)
expf(4518)
expf(4519)
expf(4520)
expf(4521)
expf(4522)
expf(4523)
expf(4524)
expf(4525)
expf(4526)
expf(4527)
expf(4528)
expf(4529)
expf(4530)
expf(4531)
expf(4532)
expf(4533)
expf(4534)
expf(4535)
expf(4536)
expf(4537)
expf(4538)
expf(4539)
expf(4540)
expf(4541)
expf(4542)
expf(4543)
expf(4544)
expf(4545)
expf(4546)
expf(4547)
expf(4548)
expf(4549)
expf(4550)
expf(4551)
expf(4552)
expf(4553)
expf(4554)
expf(4555)
expf(4556)
expf(4557)
expf(4558)
expf(4559)
expf(4560)
expf(4561)
expf(4562)
expf(4563)
expf(4564)
expf(4565)
expf(4566)
expf(4567)
expf(4568)
expf(4569)
expf(4570)
expf(4571)
expf(4572)
expf(4573)
expf(4574)
expf(4575)
expf(4576)
expf(4577)
expf(4578)
expf(4579)
expf(4580)
expf(4581)
expf(4582)
expf(4583)
expf(4584)
expf(4585)
expf(4586)
expf(4587)
expf(4588)
expf(4589)
expf(4590)
expf(4591)
expf(4592)
expf(4593)
expf(4594)
expf(4595)
expf(4596)
expf(4597)
expf(4598)
expf(4599)
expf(4600)
expf(4601)
expf(4602)
expf(4603)
expf(4604)
expf(4605)
expf(4606)
expf(4607)
expf(4608)
expf(4609)
expf(4610)
expf(4611)
expf(4612)
expf(4613)
expf(4614)
expf(4615)
expf(4616)
expf(4617)
expf(4618)
expf(4619)
expf(4620)
expf(4621)
expf(4622)
expf(4623)
expf(4624)
expf(4625)
expf(4626)
expf(4627)
expf(4628)
expf(4629)
expf(4630)
expf(4631)
expf(4632)
expf(4633)
expf(4634)
expf(4635)
expf(4636)
expf(4637)
expf(4638)
expf(4639)
expf(4640)
expf(4641)
expf(4642)
expf(4643)
expf(4644)
expf(4645)
expf(4646)
expf(4647)
expf(4648)
expf(4649)
expf(4650)
expf(4651)
expf(4652)
expf(4653)
expf(4654)
expf(4655)
expf(4656)
expf(4657)
expf(4658)
expf(4659)
expf(4660)
expf(4661)
expf(4662)
expf(4663)
expf(4664)
expf(4665)
expf(4666)
expf(4667)
expf(4668)
expf(4669)
expf(4670)
expf(4671)
expf(4672)
expf(4673)
expf(4674)
expf(4675)
expf(4676)
expf(4677)
expf(4678)
expf(4679)
expf(4680)
expf(4681)
expf(4682)
expf(4683)
expf(4684)
expf(4685)
expf(4686)
expf(4687)
expf(4688)
expf(4689)
expf(4690)
expf(4691)
expf(4692)
expf(4693)
expf(4694)
expf(4695)
expf(4696)
expf(4697)
expf(4698)
expf(4699)
expf(4700)
expf(4701)
expf(4702)
expf(4703)
expf(4704)
expf(4705)
expf(4706)
expf(4707)
expf(4708)
expf(4709)
expf(4710)
expf(4711)
expf(4712)
expf(4713)
expf(4714)
expf(4715)
expf(4716)
expf(4717)
expf(4718)
expf(4719)
expf(4720)
expf(4721)
expf(4722)
expf(4723)
expf(4724)
expf(4725)
expf(4726)
expf(4727)
expf(4728)
expf(4729)
expf(4730)
expf(4731)
expf(4732)
expf(4733)
expf(4734)
expf(4735)
expf(4736)
expf(4737)
expf(4738)
expf(4739)
expf(4740)
expf(4741)
expf(4742)
expf(4743)
expf(4744)
expf(4745)
expf(4746)
expf(4747)
expf(4748)
expf(4749)
expf(4750)
expf(4751)
expf(4752)
expf(4753)
expf(4754)
expf(4755)
expf(4756)
expf(4757)
expf(4758)
expf(4759)
expf(4760)
expf(4761)
expf(4762)
expf(4763)
expf(4764)
expf(4765)
expf(4766)
expf(4767)
expf(4768)
expf(4769)
expf(4770)
expf(4771)
expf(4772)
expf(4773)
expf(4774)
expf(4775)
expf(4776)
expf(4777)
expf(4778)
expf(4779)
expf(4780)
expf(4781)
expf(4782)
expf(4783)
expf(4784)
expf(4785)
expf(4786)
expf(4787)
expf(4788)
expf(4789)
expf(4790)
expf(4791)
expf(4792)
expf(4793)
expf(4794)
expf(4795)
expf(4796)
expf(4797)
expf(4798)
expf(4799)
expf(4800)
expf(4801)
expf(4802)
expf(4803)
expf(4804)
expf(4805)
expf(4806)
expf(4807)
expf(4808)
expf(4809)
expf(4810)
expf(4811)
expf(4812)
expf(4813)
expf(4814)
expf(4815)
expf(4816)
expf(4817)
expf(4818)
expf(4819)
expf(4820)
expf(4821)
expf(4822)
expf(4823)
expf(4824)
expf(4825)
expf(4826)
expf(4827)
expf(4828)
expf(4829)
expf(4830)
expf(4831)
expf(4832)
expf(4833)
expf(4834)
expf(4835)
expf(4836)
expf(4837)
expf(4838)
expf(4839)
expf(4840)
expf(4841)
expf(4842)
expf(4843)
expf(4844)
expf(4845)
expf(4846)
expf(4847)
expf(4848)
expf(4849)
expf(4850)
expf(4851)
expf(4852)
expf(4853)
expf(4854)
expf(4855)
expf(4856)
expf(4857)
expf(4858)
expf(4859)
expf(4860)
expf(4861)
expf(4862)
expf(4863)
expf(4864)
expf(4865)
expf(4866)
expf(4867)
expf(4868)
expf(4869)
expf(4870)
expf(4871)
expf(4872)
expf(4873)
expf(4874)
expf(4875)
expf(4876)
expf(4877)
expf(4878)
expf(4879)
expf(4880)
expf(4881)
expf(4882)
expf(4883)
expf(4884)
expf(4885)
expf(4886)
expf(4887)
expf(4888)
expf(4889)
expf(4890)
expf(4891)
expf(4892)
expf(4893)
expf(4894)
expf(4895)
expf(4896)
expf(4897)
expf(4898)
expf(4899)
expf(4900)
expf(4901)
expf(4902)
expf(4903)
expf(4904)
expf(4905)
expf(4906)
expf(4907)
expf(4908)
expf(4909)
expf(4910)
expf(4911)
expf(4912)
expf(4913)
expf(4914)
expf(4915)
expf(4916)
expf(4917)
expf(4918)
expf(4919)
expf(4920)
expf(4921)
expf(4922)
expf(4923)
expf(4924)
expf(4925)
expf(4926)
expf(4927)
expf(4928)
expf(4929)
expf(4930)
expf(4931)
expf(4932)
expf(4933)
expf(4934)
expf(4935)
expf(4936)
expf(4937)
expf(4938)
expf(4939)
expf(4940)
expf(4941)
expf(4942)
expf(4943)
expf(4944)
expf(4945)
expf(4946)
expf(4947)
expf(4948)
expf(4949)
expf(4950)
expf(4951)
expf(4952)
expf(4953)
expf(4954)
expf(4955)
expf(4956)
expf(4957)
expf(4958)
expf(4959)
expf(4960)
expf(4961)
expf(4962)
expf(4963)
expf(4964)
expf(4965)
expf(4966)
expf(4967)
expf(4968)
expf(4969)
expf(4970)
expf(4971)
expf(4972)
expf(4973)
expf(4974)
expf(4975)
expf(4976)
expf(4977)
expf(4978)
expf(4979)
expf(4980)
expf(4981)
expf(4982)
expf(4983)
expf(4984)
expf(4985)
expf(4986)
expf(4987)
expf(4988)
expf(4989)
expf(4990)
expf(4991)
expf(4992)
expf(4993)
expf(4994)
expf(4995)
expf(4996)
expf(4997)
expf(4998)
expf(4999)
#endif
